/*****************************************************************************
 * davinci.h: decoder and encoder modules using the DaVinci DSP.
 *****************************************************************************
 * Copyright (C) 2008 M2X BV
 * $Id: 6a2ed91bcb0efefc6cda4eb3b31a0e14f6de663f $
 *
 * Authors: Antoine Cellerier <dionoea at videolan dot org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef VLC_DAVINCI_H
#define VLC_DAVINCI_H

#include <vlc/vlc.h>
#include <vlc/decoder.h>

#define XDM_INCLUDE_DOT9_SUPPORT /* Support old 0.9 API */
#include <gnu/targets/std.h>
#include <xdc/std.h>
#include <ti/xdais/xdas.h>
#include <ti/xdais/dm/xdm.h>
#include <ti/sdo/ce/CERuntime.h>
#include <ti/sdo/ce/Engine.h>

/* #define DEBUG_DAVINCI */

int OpenVideoDecoder( vlc_object_t * );
void CloseVideoDecoder( vlc_object_t * );

int OpenAudioDecoder( vlc_object_t * );
void CloseAudioDecoder( vlc_object_t * );

int OpenVideoEncoder( vlc_object_t * );
void CloseVideoEncoder( vlc_object_t * );

int __AllocateBuffer( vlc_object_t *, XDAS_Int32, XDAS_Int32 *, XDM_BufDesc * );
#define AllocateBuffer( a, b, c, d ) __AllocateBuffer( VLC_OBJECT( a ), b, c, d )
void FreeBuffer( XDM_BufDesc * );
const char *davinci_GetExtendedError( XDAS_Int32 );
void __PrintAvailableAlgorithms( vlc_object_t *, const char * );
#define PrintAvailableAlgorithms( a, b ) __PrintAvailableAlgorithms( VLC_OBJECT( a ), b )
XDAS_Int32 VlcChromaToXdm( vlc_fourcc_t );

/*****************************************************************************
 * Common stuff
 *****************************************************************************/
extern const char *ppsz_engine_error[];

/* FOURCC codes copied from libavcodec module */
#define CASE_MPEG1 \
        case VLC_FOURCC('m','p','e','g'): \
        case VLC_FOURCC('m','p','g','1'): \
        case VLC_FOURCC('P','I','M','1'):

#define CASE_MPEG2 \
        case VLC_FOURCC('m','p','g','v'): \
        case VLC_FOURCC('m','p','2','v'): \
        case VLC_FOURCC('M','P','E','G'): \
        case VLC_FOURCC('m','p','g','2'): \
        case VLC_FOURCC('h','d','v','1'): \
        case VLC_FOURCC('h','d','v','2'): \
        case VLC_FOURCC('h','d','v','3'): \
        case VLC_FOURCC('h','d','v','5'): \
        case VLC_FOURCC('m','x','5','n'): \
        case VLC_FOURCC('m','x','5','p'): \
        case VLC_FOURCC('m','x','4','n'): \
        case VLC_FOURCC('m','x','4','p'): \
        case VLC_FOURCC('m','x','3','n'): \
        case VLC_FOURCC('m','x','3','p'): \
        case VLC_FOURCC('x','d','v','2'): \
        case VLC_FOURCC('A','V','m','p'): \
        case VLC_FOURCC('V','C','R','2'): \
        case VLC_FOURCC('M','M','E','S'): \
        case VLC_FOURCC('m','m','e','s'):

#define CASE_MPEG4 \
        case VLC_FOURCC('D','I','V','X'): \
        case VLC_FOURCC('d','i','v','x'): \
        case VLC_FOURCC('M','P','4','S'): \
        case VLC_FOURCC('M','P','4','s'): \
        case VLC_FOURCC('M','4','S','2'): \
        case VLC_FOURCC('m','4','s','2'): \
        case VLC_FOURCC('x','v','i','d'): \
        case VLC_FOURCC('X','V','I','D'): \
        case VLC_FOURCC('X','v','i','D'): \
        case VLC_FOURCC('X','V','I','X'): \
        case VLC_FOURCC('x','v','i','x'): \
        case VLC_FOURCC('D','X','5','0'): \
        case VLC_FOURCC('d','x','5','0'): \
        case VLC_FOURCC('B','L','Z','0'): \
        case VLC_FOURCC('B','X','G','M'): \
        case VLC_FOURCC('m','p','4','v'): \
        case VLC_FOURCC('M','P','4','V'): \
        case VLC_FOURCC( 4 , 0 , 0 , 0 ): \
        case VLC_FOURCC('m','4','c','c'): \
        case VLC_FOURCC('M','4','C','C'): \
        case VLC_FOURCC('F','M','P','4'): \
        case VLC_FOURCC('f','m','p','4'): \
        case VLC_FOURCC('3','I','V','2'): \
        case VLC_FOURCC('3','i','v','2'): \
        case VLC_FOURCC('U','M','P','4'): \
        case VLC_FOURCC('W','V','1','F'): \
        case VLC_FOURCC('S','E','D','G'): \
        case VLC_FOURCC('R','M','P','4'): \
        case VLC_FOURCC('H','D','X','4'): \
        case VLC_FOURCC('h','d','x','4'): \
        case VLC_FOURCC('S','M','P','4'): \
        case VLC_FOURCC('f','v','f','w'): \
        case VLC_FOURCC('F','V','F','W'): \
        /* MSMPEG4 v1 (Not sure that we should include these) */ \
        case VLC_FOURCC('D','I','V','1'): \
        case VLC_FOURCC('d','i','v','1'): \
        case VLC_FOURCC('M','P','G','4'): \
        case VLC_FOURCC('m','p','g','4'): \
        /* MSMPEG4 v2 (Not sure that we should include these) */ \
        case VLC_FOURCC('D','I','V','2'): \
        case VLC_FOURCC('d','i','v','2'): \
        case VLC_FOURCC('M','P','4','2'): \
        case VLC_FOURCC('m','p','4','2'): \
        /* MSMPEG4 v3 (Not sure that we should include these) */ \
        case VLC_FOURCC('M','P','G','3'): \
        case VLC_FOURCC('m','p','g','3'): \
        case VLC_FOURCC('d','i','v','3'): \
        case VLC_FOURCC('M','P','4','3'): \
        case VLC_FOURCC('m','p','4','3'): \
        case VLC_FOURCC('D','I','V','3'): \
        case VLC_FOURCC('D','I','V','4'): \
        case VLC_FOURCC('d','i','v','4'): \
        case VLC_FOURCC('D','I','V','5'): \
        case VLC_FOURCC('d','i','v','5'): \
        case VLC_FOURCC('D','I','V','6'): \
        case VLC_FOURCC('d','i','v','6'): \
        case VLC_FOURCC('C','O','L','1'): \
        case VLC_FOURCC('c','o','l','1'): \
        case VLC_FOURCC('C','O','L','0'): \
        case VLC_FOURCC('c','o','l','0'): \
        case VLC_FOURCC('A','P','4','1'): \
        case VLC_FOURCC('3','I','V','D'): \
        case VLC_FOURCC('3','i','v','d'): \
        case VLC_FOURCC('3','V','I','D'): \
        case VLC_FOURCC('3','v','i','d'):

#define CASE_H264 \
        case VLC_FOURCC('a','v','c','1'): \
        case VLC_FOURCC('A','V','C','1'): \
        case VLC_FOURCC('h','2','6','4'): \
        case VLC_FOURCC('H','2','6','4'): \
        case VLC_FOURCC('x','2','6','4'): \
        case VLC_FOURCC('X','2','6','4'): \
        case VLC_FOURCC('V','S','S','H'): \
        case VLC_FOURCC('V','S','S','W'): \
        case VLC_FOURCC('v','s','s','h'): \
        case VLC_FOURCC('D','A','V','C'): \
        case VLC_FOURCC('d','a','v','c'):

#define CASE_VC1 \
        case VLC_FOURCC('W','V','C','1'): \
        case VLC_FOURCC('w','v','c','1'):

#define CASE_MP3 \
        case VLC_FOURCC('m','p','g','a'): \
        case VLC_FOURCC('m','p','3',' '): \
        case VLC_FOURCC('.','m','p','3'): \
        case VLC_FOURCC('M','P','3',' '): \
        case VLC_FOURCC('L','A','M','E'):

#define CASE_AAC \
        case VLC_FOURCC('m','p','4','a'):

#endif
