/*****************************************************************************
 * davinci.c : DaVinci resizer module
 *****************************************************************************
 * Copyright (C) 2008 M2X BV
 * $Id: 025a928f0f1d2ad11b34355b5834a1480768b7d9 $
 *
 * Authors: Antoine Cellerier <dionoea at videolan dot org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#undef _FILE_OFFSET_BITS
#include <string.h>                                            /* strerror() */
#include <stdlib.h>                                      /* malloc(), free() */
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/mman.h>

/* FIXME: Including stuff directly from kernel is a bit ugly */
#include <asm-arm/arch-davinci/davinci_resizer.h>

#include <vlc/vlc.h>
#include <vlc/vout.h>

/* #define DAVINCI_DEBUG */

/*****************************************************************************
 * Local and extern prototypes.
 *****************************************************************************/
static int Open( vlc_object_t * );
static void Close( vlc_object_t * );

static void Convert( vout_thread_t *, picture_t *, picture_t * );
static uint8_t *GetBuffer( vout_thread_t *p_vout, int buf_type, int size, int *offset );

struct chroma_sys_t
{
    int i_fd; /* /dev/davinci_resizer */

    rsz_resize_t resize;

    uint8_t *p_in;
    int i_in_size;
    uint8_t *p_out;
    int i_out_size;

    vlc_bool_t b_inited;
};

/*****************************************************************************
 * Module descriptor.
 *****************************************************************************/
vlc_module_begin();
    set_description( _("Conversions using /dev/davinci_resizer") );
    set_capability( "chroma", 200 );
    set_callbacks( Open, Close );
vlc_module_end();

/*****************************************************************************
 *
 *****************************************************************************/
static int Open( vlc_object_t *p_this )
{
    vout_thread_t *p_vout = (vout_thread_t *)p_this;

    switch( p_vout->render.i_chroma )
    {
        /*case VLC_FOURCC('Y','V','1','2'):
        case VLC_FOURCC('I','4','2','0'):
        case VLC_FOURCC('I','Y','U','V'):*/
        case VLC_FOURCC('U','Y','V','Y'):
        case VLC_FOURCC('U','Y','N','V'):
        case VLC_FOURCC('Y','4','2','2'):
            switch( p_vout->output.i_chroma )
            {
                case VLC_FOURCC('U','Y','V','Y'):
                case VLC_FOURCC('U','Y','N','V'):
                case VLC_FOURCC('Y','4','2','2'):
                    p_vout->chroma.pf_convert = Convert;
                    break;

                default:
                    return VLC_EGENERIC;
            }
            break;

        default:
            return VLC_EGENERIC;
    }

    p_vout->chroma.p_sys = malloc( sizeof( chroma_sys_t ) );
    if( !p_vout->chroma.p_sys )
        return VLC_ENOMEM;

    chroma_sys_t *p_sys = p_vout->chroma.p_sys;
    memset( p_sys, 0, sizeof( chroma_sys_t ) );

    p_sys->i_fd = open( "/dev/davinci_resizer", O_RDWR );
    if( p_sys->i_fd == -1 )
    {
        msg_Err( p_vout, "Failed to open /dev/davinci_resizer (%m)." );
        goto err;
    }

    msg_Info( p_vout, "/dev/davinci_resizer opened (using buffers)" );

    p_sys->b_inited = VLC_FALSE;

    return VLC_SUCCESS;

    err:
        if( p_sys->i_fd != -1 ) close( p_sys->i_fd );
        free( p_sys );
        return VLC_EGENERIC;
}

static void Close( vlc_object_t *p_this )
{
    vout_thread_t *p_vout = (vout_thread_t *)p_this;
    chroma_sys_t *p_sys = p_vout->chroma.p_sys;
    close( p_sys->i_fd );
    free( p_sys );
}


/* Following functions are local */
/*****************************************************************************
 *
 *****************************************************************************/
static uint8_t *GetBuffer( vout_thread_t *p_vout, int buf_type, int size,
                           int *offset )
{
    chroma_sys_t *p_sys = p_vout->chroma.p_sys;
    uint8_t *p_buf;
    rsz_reqbufs_t reqbufs;
    rsz_buffer_t buffer;
    memset( &reqbufs, 0, sizeof( reqbufs ) );
    memset( &buffer, 0, sizeof( buffer ) );

    reqbufs.buf_type = buf_type;
    reqbufs.size = size;
    reqbufs.count = 1;

    msg_Dbg( p_vout, "Asking %s for buffers: size 0x%x, count %d",
             buf_type == RSZ_BUF_IN ? "input" : "output",
             reqbufs.size, reqbufs.count );
    if( ioctl( p_sys->i_fd, RSZ_REQBUF, &reqbufs ) )
    {
        msg_Err( p_vout, "RSZ_REQBUF failed (%m)" );
        return NULL;
    }

    buffer.buf_type = buf_type;
    buffer.index = 0;

    if( ioctl( p_sys->i_fd, RSZ_QUERYBUF, &buffer ) )
    {
        msg_Err( p_vout, "RSZ_QUERYBUF failed (%m)" );
        return NULL;
    }

    msg_Dbg( p_vout, "Got buffer: index %d, offset 0x%x",
             buffer.index, buffer.offset );
    if( offset ) *offset = buffer.offset;

    p_buf = mmap( NULL, reqbufs.size, PROT_READ|PROT_WRITE, MAP_SHARED,
                  p_sys->i_fd, buffer.offset );

    if( p_buf == MAP_FAILED )
    {
        msg_Err( p_vout, "mmap to buffer failed (%m)" );
        return NULL;
    }

    return p_buf;
}


/*****************************************************************************
 *
 *****************************************************************************/
#if 0
static double u( double s )
{
    if( s < 0 ) s = -s;
    if( 0. <= s && s <= 1. )
        return 3./2.*s*s*s-5./2.*s*s+1.;
    else if( s <= 2. )
        return -1./2.*s*s*s+5./2.*s*s-4.*s+2.;
    else
        return 0.;
}
#endif

static void Convert( vout_thread_t *p_vout, picture_t *p_source,
                                            picture_t *p_dest )
{
    chroma_sys_t *p_sys = p_vout->chroma.p_sys;

    if( !p_sys->b_inited )
    {
        int i;

        /**
         * Get the input and output buffers
         */
        if( !p_sys->p_in )
        {
            p_sys->i_in_size = p_source->p->i_pitch * p_source->p->i_visible_lines;//p_vout->render.i_width*p_vout->render.i_height*2;
            p_sys->p_in = GetBuffer( p_vout, RSZ_BUF_IN, p_sys->i_in_size, &p_sys->resize.in_buf.offset );
            p_sys->resize.in_buf.index = 0;
            if( !p_sys->p_in ) return;
        }

        if( !p_sys->p_out )
        {
            p_sys->i_out_size = p_dest->p->i_pitch * p_dest->p->i_visible_lines;
            p_sys->p_out = GetBuffer( p_vout, RSZ_BUF_OUT, p_sys->i_out_size, &p_sys->resize.out_buf.offset );
            p_sys->resize.out_buf.index = 0;
            if( !p_sys->p_out ) return;
        }

        /**
         * Set the parameters
         */
        rsz_params_t params;
        memset( &params, 0, sizeof( params ) );
        params.in_hsize = p_source->p->i_visible_pitch / 2;
        params.in_vsize = p_source->p->i_visible_lines;
        params.in_pitch = p_source->p->i_pitch;
        params.inptyp = RSZ_INTYPE_YCBCR422_16BIT;
        params.pix_fmt = RSZ_PIX_FMT_YUYV;
        params.out_hsize = p_dest->p->i_visible_pitch / 2;
        params.out_vsize = p_dest->p->i_visible_lines;
        params.out_pitch = p_dest->p->i_pitch;

#if 0
        /* Bicubic interpolation coefficients as defined in spraai7a */
        for( i = 0; i < 8; i++ )
        {
            double n = ((double)i)/8.;
            params.hfilt_coeffs[4*i] =
            params.vfilt_coeffs[4*i] = (int)(u(n+1.)*256.);
            params.hfilt_coeffs[4*i+1] =
            params.vfilt_coeffs[4*i+1] = (int)(u(n)*256.);
            params.hfilt_coeffs[4*i+2] =
            params.vfilt_coeffs[4*i+2] = (int)(u(1.-n)*256.);
            params.hfilt_coeffs[4*i+3] =
            params.vfilt_coeffs[4*i+3] = (int)(u(2.-i)*256.);
        }
#endif
        const short coefs[] = {
#if 0
            0, 256, 0, 0,
            -12, 246, 23, -1,
            -18, 222, 58, -6,
            -18, 186, 99, -11,
            -16, 144, 144, -16,
            -11, 99, 186, -18,
            -6, 58, 222, -18,
            -1, 23, 246, -12 };
#else
            0, 256, 0, 0,
            -11, 245, 23, -1,
            -17, 220, 58, -5,
            -18, 184, 100, -10,
            -15, 143, 143, -15,
            -10, 100, 184, -18,
            -5, 58, 220, -17,
            -1, 23, 245, -11 };
#endif

        for( i = 0; i < 32; i++ )
            params.hfilt_coeffs[i] = params.vfilt_coeffs[i] = coefs[i];
        params.yenh_params.type = RSZ_YENH_DISABLE;

        if( ioctl( p_sys->i_fd, RSZ_S_PARAM, &params ) )
        {
            msg_Err( p_vout, "Failed setting resizer parameters (%m)" );
            return;
        }

        /**
         * Set resizer speed to fastest
         */
        int i_speed = 0;
        if( ioctl( p_sys->i_fd, RSZ_S_EXP, &i_speed ) )
        {
            msg_Err( p_vout, "Failed setting resizer speed (%m)" );
            return;
        }

        /**
         * Initialize resize structure
         */
        p_sys->resize.in_buf.size = p_sys->i_in_size;
        p_sys->resize.out_buf.size = p_sys->i_out_size;

        /**
         * We're done
         */
        msg_Info( p_vout, "davinci resizer initialized correctly (I hope...)" );
        p_sys->b_inited = VLC_TRUE;
    }

#ifdef DAVINCI_DEBUG
    msg_Dbg( p_vout, "About to resize..." );
#endif
    memcpy( p_sys->p_in, p_source->p->p_pixels, p_sys->i_in_size );
    if( ioctl( p_sys->i_fd, RSZ_RESIZE, &p_sys->resize ) )
    {
        msg_Err( p_vout, "Resize request failed." );
        return;
    }
#ifdef DAVINCI_DEBUG
    msg_Dbg( p_vout, "Looks like resize worked!" );
#endif
    memcpy( p_dest->p->p_pixels, p_sys->p_out, p_sys->i_out_size );
#ifdef DAVINCI_DEBUG
    msg_Dbg( p_vout, "Memcpy too!" );
#endif
}
